// ****************************************** //
// *                                        * //
// *  TmsDBTreeView demo project            * //
// *                                        * //
// *  Copyright (c) mSunSoftware. 2006.     * //
// *  All rights reserved.                  * //
// *  Web:     http://www.msunsoftware.com  * //
// *  E-mail:  support@msunsoftware.com     * //
// *                                        * //
// ****************************************** //

#include <vcl.h>
#pragma hdrstop
#include "UFormMain.h"
#include "UFormFind.h"
#include "UFormMessage.h"
#include "UFormAbout.h"
#pragma package(smart_init)
#pragma link "msBaseTreeView"
#pragma link "msTreeView"
#pragma link "msDBTreeView"
#pragma resource "*.dfm"

TFormMain *FormMain;

// ===================================================================== //
MESSAGE __fastcall TFormMain::smFindNext(TMessage& Msg)
{
  TmsBaseTreeNode* Node;
  Node = msTV->FindNextNode();
  if (Node != NULL)
    msTV->Selected = Node;
  Msg.Result = int(Node != NULL);

  if (Msg.Result == 0)
    CreateMessageDlg("Search is completed!", mtInformation);

  return Msg.Result;
}

// ===================================================================== //
void __fastcall TFormMain::SetConnection()
{
  const AnsiString sConnStr      = "Provider=%s;Data Provider=%s;Data Source=%s";
  const AnsiString sProvider     = "MSDataShape.1";
  const AnsiString sDataProvider = "Microsoft.Jet.OLEDB.4.0";
  AnsiString sDataSource;

  if (ADOConnection->Connected)
    ADOConnection->Close();

  // Init the connection string
  sDataSource = ExtractFilePath(Application->ExeName) + "Database.mdb";
  ADOConnection->ConnectionString = Format(sConnStr, ARRAYOFCONST((sProvider, sDataProvider, sDataSource)));
  ADODataSet->CommandText = Format("SELECT * FROM dbTable WHERE LangID=%d", ARRAYOFCONST((FLangID)));

  // Open connection
  try {
    ADOConnection->Open();
    ADODataSet->Open();
    aConnConnected->Checked = true;
  }
  catch(...) {
    CreateMessageDlg("The database was not found or an error was fixed!", mtError);
    Application->Terminate(); // DB not found! - terminate the program
  }
}

// ===================================================================== //
void __fastcall TFormMain::SetLanguage()
{
  if (FLangID == 0)
  {
    mMenu_File->Caption         = "File";
    aFileLoad->Caption          = "Load Tree";
    aFileSave->Caption          = "Save Tree";
    aFileFullExpand->Caption    = "Full Expand";
    aFileFullCollapse->Caption  = "Full Collapse";
    aFileExit->Caption          = "Close";
    mMenu_Conn->Caption         = "Connection";
    aConnConnected->Caption     = "Connected";
    mMenu_Edit->Caption         = "Edit";
    aEditAdd->Caption           = "Add Node";
    aEditDelete->Caption        = "Delete Node";
    aEditCut->Caption           = "Cut";
    aEditCopy->Caption          = "Copy";
    aEditPaste->Caption         = "Paste";
    aEditRename->Caption        = "Rename Node";
    aEditFind->Caption          = "Find";
    mMenu_Tools->Caption        = "Tools";
    aTools_AutoDrag->Caption    = "Auto Drag";
    aTools_AutoSort->Caption    = "Auto Sort";
    aTools_SortAZ->Caption      = "Sort A-Z";
    aTools_SortZA->Caption      = "Sort Z-A";
    aTools_ReadOnly->Caption    = "Read Only";
    mMenu_Lang->Caption         = "Language";
    aLangEng->Caption           = "English";
    aLangRus->Caption           = "Russian";
    mMenu_Help->Caption         = "Help";
    aHelpAbout->Caption         = "About";
    gBox_Data->Caption          = "The Data of the field \"Description\"";
    gBox_DataSet->Caption       = "DataSet";
  }
  else
  {
    mMenu_File->Caption         = "";
    aFileLoad->Caption          = " ";
    aFileSave->Caption          = " ";
    aFileFullExpand->Caption    = "  ";
    aFileFullCollapse->Caption  = "  ";
    aFileExit->Caption          = "";
    mMenu_Conn->Caption         = "";
    aConnConnected->Caption     = "";
    mMenu_Edit->Caption         = "";
    aEditAdd->Caption           = " ";
    aEditDelete->Caption        = " ";
    aEditCut->Caption           = "";
    aEditCopy->Caption          = "";
    aEditPaste->Caption         = "";
    aEditRename->Caption        = " ";
    aEditFind->Caption          = "";
    mMenu_Tools->Caption        = "";
    aTools_AutoDrag->Caption    = " ";
    aTools_AutoSort->Caption    = " ";
    aTools_SortAZ->Caption      = "  A  Z";
    aTools_SortZA->Caption      = "  Z  A";
    aTools_ReadOnly->Caption    = " ";
    mMenu_Lang->Caption         = "";
    aLangEng->Caption           = "";
    aLangRus->Caption           = "";
    mMenu_Help->Caption         = "";
    aHelpAbout->Caption         = " ";
    gBox_Data->Caption          = "  \"\"";
    gBox_DataSet->Caption       = " ";
  }
}

// ===================================================================== //
__fastcall TFormMain::TFormMain(TComponent* Owner)
  : TForm(Owner)
{
}

// ===================================================================== //
void __fastcall TFormMain::FormCreate(TObject *Sender)
{
  FLangID = 0;
  // Open connection to database
  SetConnection();
  // Set language
  SetLanguage();

  // Init menu items
  aEditAdd->ShortCut    = msTV->TVShortCuts->scAddNode;
  aEditDelete->ShortCut = msTV->TVShortCuts->scDeleteNode;
  aEditCut->ShortCut    = msTV->TVShortCuts->scCutNode;
  aEditCopy->ShortCut   = msTV->TVShortCuts->scCopyNode;
  aEditPaste->ShortCut  = msTV->TVShortCuts->scPasteNode;
  aEditRename->ShortCut = msTV->TVShortCuts->scEditNode;
  aEditFind->ShortCut   = msTV->TVShortCuts->scFindNode;

  aConnConnected->Checked  = ADODataSet->Active;
  aTools_AutoDrag->Checked = msTV->TVOptions->AutoDrag;
  aTools_AutoSort->Checked = msTV->TVOptions->AutoSort;
  aTools_ReadOnly->Checked = msTV->ReadOnly;
  aLangEng->Checked = true;

  WindowState = wsMaximized;

  // Init TreeView
  msTV->SortTreeNodes(true);
}

// ===================================================================== //
void __fastcall TFormMain::aFileLoadExecute(TObject *Sender)
{
  if ( OpenDlg->Execute() )
    msTV->LoadFromFile(OpenDlg->FileName);
}

// ===================================================================== //
void __fastcall TFormMain::aFileSaveExecute(TObject *Sender)
{
  if ( SaveDlg->Execute() )
    msTV->SaveToFile(SaveDlg->FileName);
}

// ===================================================================== //
void __fastcall TFormMain::aFileFullExpandExecute(TObject *Sender)
{
  msTV->Items->BeginUpdate();
  try {
    msTV->FullExpand();
  }
  __finally {
    msTV->Items->EndUpdate();
  }
}

// ===================================================================== //
void __fastcall TFormMain::aFileFullCollapseExecute(TObject *Sender)
{
  msTV->Items->BeginUpdate();
  try {
    msTV->FullCollapse();
  }
  __finally {
    msTV->Items->EndUpdate();
  }
}

// ===================================================================== //
void __fastcall TFormMain::aFileExitExecute(TObject *Sender)
{
  Close();
}

// ===================================================================== //
void __fastcall TFormMain::aConnConnectedExecute(TObject *Sender)
{
  aConnConnected->Checked = !aConnConnected->Checked;
  if (aConnConnected->Checked)
    SetConnection();
  else
    ADOConnection->Close();
}

// ===================================================================== //
void __fastcall TFormMain::aEditAddExecute(TObject *Sender)
{
  if (msTV->Focused() && (!msTV->IsEditing()))
    msTV->AddNode(NULL);
}

// ===================================================================== //
void __fastcall TFormMain::aEditDeleteExecute(TObject *Sender)
{
  if (msTV->Focused() && (!msTV->IsEditing()))
    msTV->DeleteNode(NULL);
}

// ===================================================================== //
void __fastcall TFormMain::aEditCutExecute(TObject *Sender)
{
  if (msTV->Focused() && (!msTV->IsEditing()))
    msTV->CutNode(NULL);
}

// ===================================================================== //
void __fastcall TFormMain::aEditCopyExecute(TObject *Sender)
{
  if (msTV->Focused() && (!msTV->IsEditing()))
    msTV->CopyNode(NULL);
}

// ===================================================================== //
void __fastcall TFormMain::aEditPasteExecute(TObject *Sender)
{
  if (msTV->Focused() && (!msTV->IsEditing()))
    msTV->PasteNode(NULL);
}

// ===================================================================== //
void __fastcall TFormMain::aEditRenameExecute(TObject *Sender)
{
  if (msTV->Focused() && (!msTV->IsEditing()))
    msTV->EditNode(NULL);
}

// ===================================================================== //
void __fastcall TFormMain::aEditFindExecute(TObject *Sender)
{
  if (msTV->Focused() && (!msTV->IsEditing()))
    msTV->InitSearchParams();
}

// ===================================================================== //
void __fastcall TFormMain::aTools_AutoDragExecute(TObject *Sender)
{
  aTools_AutoDrag->Checked = !aTools_AutoDrag->Checked;
  msTV->TVOptions->AutoDrag = aTools_AutoDrag->Checked;
}

// ===================================================================== //
void __fastcall TFormMain::aTools_AutoSortExecute(TObject *Sender)
{
  aTools_AutoSort->Checked = !aTools_AutoSort->Checked;
  msTV->TVOptions->AutoSort = aTools_AutoSort->Checked;
}

// ===================================================================== //
void __fastcall TFormMain::aTools_SortAZExecute(TObject *Sender)
{
  msTV->TVOptions->SortDirection = sdAlphabeticOrder;
}

// ===================================================================== //
void __fastcall TFormMain::aTools_SortZAExecute(TObject *Sender)
{
  msTV->TVOptions->SortDirection = sdInvertedOrder;
}

// ===================================================================== //
void __fastcall TFormMain::aTools_ReadOnlyExecute(TObject *Sender)
{
  aTools_ReadOnly->Checked = !aTools_ReadOnly->Checked;
  msTV->ReadOnly = aTools_ReadOnly->Checked;
}

// ===================================================================== //
void __fastcall TFormMain::aLangEngExecute(TObject *Sender)
{
  ((TAction*)Sender)->Checked = true;
  int ID = ((TComponent*)Sender)->Tag;
  if (ID == 0) // English
    aLangRus->Checked = false;
  else         // Russian;
    aLangEng->Checked = false;

  // Set Language
  if (FLangID != ID)
  {
    FLangID = ID;
    SetLanguage();
  }
}

// ===================================================================== //
void __fastcall TFormMain::aHelpAboutExecute(TObject *Sender)
{
  FormAbout = new TFormAbout(this);
  FormAbout->ShowModal();
  FormAbout->Release();
}

// ===================================================================== //
void __fastcall TFormMain::msTVCollapsed(TObject *Sender,
      TmsBaseTreeNode *Node)
{
  msTV->Invalidate();
}

// ===================================================================== //
void __fastcall TFormMain::msTVGetImageIndex(TObject *Sender,
      TmsBaseTreeNode *Node)
{
  if (!Node->HasChildren)
    Node->ImageIndex = 0;
  else
  {
    if (!Node->Expanded) 
      Node->ImageIndex = 1;
    else
      Node->ImageIndex = 2;
  }
}

// ===================================================================== //
void __fastcall TFormMain::msTVGetSelectedIndex(TObject *Sender,
      TmsBaseTreeNode *Node)
{
  Node->SelectedIndex = Node->ImageIndex;
}

// ===================================================================== //
void __fastcall TFormMain::msTVBeforeSearchNode(TObject *Sender,
      TmsSearchNodeParams &SearchNodeParams)
{
  TPoint Point;

  FormFind = new TFormFind(this, &SearchNodeParams, FLangID);
  Point.x = ToolBar->Left + ToolBar->Width - FormFind->Width - 1;
  Point.y = ToolBar->Top + ToolBar->Height + 1;
  Point = ClientToScreen(Point);
  FormFind->Left = Point.x;
  FormFind->Top = Point.y;
  FormFind->ShowModal();
  FormFind->Release();
}

// ===================================================================== //
void __fastcall TFormMain::msTVMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
  TmsBaseTreeNode* Node;
  TPoint Point;

  if (Button == mbRight)
  {
    Node = msTV->GetNodeAt(X, Y);
    if (Node != NULL)
    {
      msTV->Selected = Node;
      Point.x = X;
      Point.y = Y;
      Point = msTV->ClientToScreen(Point);
      msTV->PopupMenu->Popup(Point.x, Point.y);
    }
  }
}

// ===================================================================== //
void __fastcall TFormMain::msTVProcessError(TObject *Sender,
      AnsiString Msg)
{
  CreateMessageDlg(Msg, mtWarning);
}

// ===================================================================== //
void __fastcall TFormMain::ADODataSetAfterInsert(TDataSet *DataSet)
{
  DataSource->DataSet->FieldByName("LangID")->AsInteger = FLangID;

}

